#!/usr/bin/perl -w

use Jcode;

print "Reading ...\n";
open K, ">list.kana";
<>; # skip ID line
while(<>) {
	chomp;

	/([^ ]+) (\[[^\]]+\] )?(.*)/ || print "$_\n";

	$eng = $3;
	if($2 && $2 ne "") {
		$kanji = $1;
		$kana = $2;
		$kana =~ /\[(.*)\]/;
		$kana = $1;
		
	} else {
		$kana = $1;
		$kanji = "";
	}
	@list = split /\//, $eng;
	
	$kana = convert($kana,"SJIS","EUC-JP");
	$kanji = convert($kanji,"SJIS","EUC-JP");
	if($kanji eq $kana) { $kanji = ""; }
	
	print K "$kana\n"; # for romaji

	# [0] will always be blank, due to leading /
	shift @list;

	# Add a definition
	push @defs, join('; ', @list);

	# Associate the definition with the kana (and kanji, if any.)  Non-case-sensitive.
	$def{lc($kana)} .= $#defs . " ";
	if($kanji ne "") {
		$def{lc($kanji)} .= $#defs . " ";
	}

	# If we have a a kanji, associate it with this definition number,
	# so if we're looking up a kana and get multiple defs, we can show
	# the kanji associated with each.
	$defkanji[$#defs] = $kanji;
	$defkana[$#defs] = $kana;

	# Associate the number with each English definition.  (Why
	# can't hash values be arrays?)
	for($i = 0; $i <= $#list; $i++) {
		# Remove parenthesized stuff from the key names; this'll
		# combine stuff like /fool/ and /(an) (id) fool/.

		$orig = lc($list[$i]);
		$orig =~ s/ *\([^\)]*\) *//g;
		$orig =~ s/^ *//g;
		$orig =~ s/ *$//g;
		$key = lc($orig);
		$key =~ s/[ \.-^]+/ /g; 

		# That may have wiped out the whole def, such as "/(lit: foo)/".
		($key eq "") && next;

		# Associate this (english) key with another def index
		$eng{$key} .= $#defs . " ";

		# Also associate the key with its original form, for printing.  Sometimes
		# we'll get dupes, such as
		# as far as ... is concerned
		# as far as...is concerned
		# Babylon would kill off one of these, so we take the first.
		if(!$eng_orig{$key}) {
			$eng_orig{$key} = $orig;
		}

	}
}
close K;

# Lists: 
# $kanalist, $kanjilist: Each kana in the edict, and associated kanji (if any).
# Not iterated over, since kana can occur more than once; we need to do every
# kana and kanji exactly once (since it'll discard dupes).

# $rom: Romaji for each kana.

# $eng{english}: Associated entry in kanalist/kanjilist for each English
# word (multiples separated by spaces.)

# %def{id}: List of definitions for any given kana/kanji.  
print "Romaji ...\n";
`./kana2rom < list.kana > list.romaji`;

open K, "list.kana";
open R, "list.romaji";

print "Comb kana/romaji ...\n";
while($k = <K>) {
	$r = <R>;
	chomp $k;
	chomp $r;
	$r =~ s/(.)_/$1$1/g;
	$rom{$k} = $r;
}
close K;
close R;

unlink "list.kana";
unlink "list.romaji";

print "Output Japanese keys ...\n";
open O, ">key-japanese.txt";

sub print_def
{
	local($ind) = @_;

	print O "$rom{$defkana[$ind]}: $defkana[$ind]<br>";

	if($defkanji[$ind] ne "") {
		# We have a kanji associated with this def; print it, too.
		print O "$defkanji[$ind]<br>";
	}
	print O "$defs[$ind]";
}

# Key on J:
# One definition for every kana and kanji.  The romaji get keyed in,
# too--but only for the kana defs.  Don't combine kana and kanji, or
# try to associate a kana's romaji with the kanji, since it'll get
# put in multiple times and some will be dropped.
foreach my $key (keys %def) {
	push @defkeys, $key;
}
foreach my $key (keys %eng) {
	push @engkeys, $key;
}
@defkeys = sort @defkeys;
@engkeys = sort @engkeys;

for ($i = 0; $i < $#defkeys; $i++) {
	$key = $defkeys[$i];
	# key is either a kana or kanji; if it's kana, we'll
	# have a romaji for it, too.
	print O "$key";
	($rom{$key}) && print O "|$rom{$key}";
	print O "\n";

	# $key is either a kana or a kanji; $def{$key} is the associated
	# definitions for it.
	$first = 1;
	foreach my $ind (split / /, $def{$key}) {
		if($first) {
			$first = 0;
		} else {
			print O "<hr>";
		}
		print_def($ind);
	}
	print O "\n\n";
}

close O;

print "Output English keys ...\n";
open O, ">key-eng.txt";

# Key on E:
# foreach my $key (keys %eng) {
for ($i = 0; $i < $#engkeys; $i++) {
	$key = $engkeys[$i];
	# Key is the English key.
	print O "$eng_orig{$key}\n";

	# Print every definition that's associated with this key.
	$first = 1;
	foreach my $ind (split / /, $eng{$key}) {
		if($first) {
			$first = 0;
		} else {
			print O "<hr>";
		}
		print_def($ind);
	}
	print O "\n\n";
}
close O;

